﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml;
using System.IO;
using System.Threading;
using System.Diagnostics;

namespace Dock
{
    /// <summary>
    /// Dock
    /// </summary>
    [Serializable]
    public class Dock
    {
        #region Events

        /// <summary>
        /// Fired when a DockItem is added to this Dock
        /// </summary>
        public event EventHandler<DockItemEventArgs> OnItemAdded;

        /// <summary>
        /// Fired when a DockItem is removed from this Dock
        /// </summary>
        public event EventHandler<DockItemEventArgs> OnItemRemoved;

        /// <summary>
        /// Fired when the Dock has been successfully saved to a file
        /// </summary>
        public event EventHandler<EventArgs> OnSaveComplete;

        #endregion

        #region Fields

        /// <summary>
        /// A collection of DockItems
        /// </summary>
        private List<DockItem> innerDockItems;

        #endregion

        #region Properties

        /// <summary>
        /// Get or set a DockItem in this Dock
        /// </summary>
        /// <param name="item"></param>
        /// <returns></returns>
        public DockItem this[int index]
        {
            get
            {
                return this.innerDockItems[index];
            }
            set
            {
                this.innerDockItems[index] = value;
            }
        }

        /// <summary>
        /// Get or set a DockItem in this Dock
        /// </summary>
        /// <param name="name"></param>
        /// <returns></returns>
        public DockItem this[string name]
        {
            get
            {
                int indexOfItem = this.GetItemIndex(name);

                if (indexOfItem >= 0)
                {
                    return this[indexOfItem];
                }

                return null;
            }
            set
            {
                // Get the index of the DockItem
                int indexOfItem = this.GetItemIndex(name);

                if (indexOfItem >= 0)
                {
                    this[indexOfItem] = value;
                }
            }
        }

        /// <summary>
        /// Get the total number of DockItems in this Dock
        /// </summary>
        public int Count
        {
            get
            {
                return this.innerDockItems.Count;
            }
        }

        /// <summary>
        /// Get if this collection is current being saved to a file
        /// </summary>
        public bool IsSaving
        {
            get;
            private set;
        }

        /// <summary>
        /// Get or set the items file name
        /// </summary>
        public string ItemsFileName
        {
            get;
            set;
        }

        /// <summary>
        /// Get or set the maximum number of items which can be contained in this DocItemCollection
        /// </summary>
        public int MaximumItems
        {
            get;
            set;
        }

        /// <summary>
        /// Get the total number of DocItems which can be added to this collection
        /// </summary>
        public int RemainingItems
        {
            get
            {
                return (this.MaximumItems - this.Count);
            }
        }

        /// <summary>
        /// Get if this DocItemCollection is full
        /// </summary>
        public bool IsFull
        {
            get
            {
                return (this.Count >= this.MaximumItems);
            }
        }

        /// <summary>
        /// Get or set if to show a popup description
        /// </summary>
        public bool ShowPopupDescription
        {
            get;
            set;
        }

        /// <summary>
        /// Get or set if the dock is always on top of other windows
        /// </summary>
        public bool AlwaysOnTop
        {
            get;
            set;
        }

        #endregion

        #region Constructors

        /// <summary>
        /// Create a new DocItemCollection
        /// </summary>
        /// <param name="maxItems"></param>
        public Dock(int maxItems)
        {
            this.innerDockItems = new List<DockItem>();
            this.MaximumItems = maxItems;

            // Create the file path for the items xml file
            this.ItemsFileName = Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData), "Dock");
            this.ItemsFileName = Path.Combine(this.ItemsFileName, "items.xml");
        }

        #endregion

        #region Methods

        /// <summary>
        /// Add a new DockItem
        /// </summary>
        /// <param name="item"></param>
        /// <returns>The DockItem added to this collection</returns>
        public DockItem Add(DockItem item)
        {
            if (this.Count < this.MaximumItems)
            {
                // Add the DockItem
                this.innerDockItems.Add(item);

                // Update the item's index
                item.Index = this.GetItemIndex(item);

                // Save the items file
                this.Save();

                // Fire the OnItemAdded event
                this.ItemAdded(item);

                // Return the added item
                return item;
            }
            else
            {
                // Throw a DocItemCollectionFull exception
                throw new DockItemCollectionFullException();
            }
        }

        /// <summary>
        /// Remove a DockItem from this collection
        /// </summary>
        /// <param name="item"></param>
        public void Remove(DockItem item)
        {
            // Remove the item
            this.innerDockItems.Remove(item);

            // Fire the OnItemRemoved event
            this.ItemRemoved(item);

            // Save the file
            this.Save();
        }
                
        /// <summary>
        /// Get an enumeration of the DockItems in this Dock
        /// </summary>
        /// <returns></returns>
        public IEnumerator<DockItem> GetEnumerator()
        {
            return this.innerDockItems.GetEnumerator();
        }

        /// <summary>
        /// Get the index number of a DockItem
        /// </summary>
        /// <param name="item"></param>
        /// <returns></returns>
        public int GetItemIndex(DockItem item)
        {
            return this.innerDockItems.IndexOf(item);
        }

        /// <summary>
        /// Get the index number of a DockItem in this Dock with the specified name
        /// </summary>
        /// <param name="fileName"></param>
        /// <returns></returns>
        public int GetItemIndex(string name)
        {
            foreach (DockItem item in this)
            {
                if (name.ToUpper() == item.Name.ToUpper())
                {
                    return this.innerDockItems.IndexOf(item);
                }
            }

            return -1;
        }

        /// <summary>
        /// Save the items in this Dock to an xml file on another thread
        /// </summary>
        public void BackgroundSave()
        {
            if (this.IsSaving)
            {
                throw new InvalidOperationException("The file is currently being saved");
            }
            else
            {
                // Debugging
                Debug.Print(DateTime.Now.ToString() + ": Background saving the items file...");

                // 
                Thread t = new Thread(new ThreadStart(this.Save));
                t.Start();
            }
        }

        /// <summary>
        /// Save the items in this DocItemCollection to an xml file
        /// </summary>
        public void Save()
        {
            // Update the IsSaving flag
            this.IsSaving = true;

            // Get the filePath
            string filePath = Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData), "Dock");

            // Create it if it doesn't exist
            if (!Directory.Exists(filePath))
            {
                Directory.CreateDirectory(filePath);
            }

            // Create a new XmlWriter to write the document items
            using (XmlWriter xw = XmlWriter.Create(this.ItemsFileName))
            {
                // 
                xw.WriteStartDocument();
                xw.WriteStartElement("dock");

                // Write the application configuration
                xw.WriteStartElement("config");
                xw.WriteElementString("alwaysOnTop", this.AlwaysOnTop.ToString());
                xw.WriteElementString("showPopupDescription", this.ShowPopupDescription.ToString());
                xw.WriteEndElement();

                // Write the items
                xw.WriteStartElement("items");

                foreach (DockItem item in this)
                {
                    xw.WriteStartElement("item");
                    xw.WriteAttributeString("number", item.Index.ToString());
                    xw.WriteAttributeString("X", item.PositionX.ToString());
                    xw.WriteElementString("fileName", item.FileName);
                    xw.WriteEndElement(); // item
                }

                xw.WriteEndElement(); // items

                xw.WriteEndElement(); // dock

                // Flush the stream, this will update the file
                xw.Flush();
            }

            // Debugging
            Debug.Print(DateTime.Now.ToString() + ": Items file saved");
            Debug.Print("");

            // Fire the OnSaveComplete event
            if (this.OnSaveComplete != null)
            {
                this.OnSaveComplete(this, new EventArgs());
            }

            // Update the IsSaving flag
            this.IsSaving = false;
        }

        /// <summary>
        /// Load items from a file
        /// </summary>
        public void Load()
        {
            // If the items file exists load it
            if (File.Exists(this.ItemsFileName))
            {
                // Load the file
                this.Load(this.ItemsFileName);
            }
        }

        /// <summary>
        /// Load items from a file
        /// </summary>
        /// <param name="itemsFileName"></param>
        public void Load(string itemsFileName)
        {
            // Clear all the items from this collection
            this.innerDockItems.Clear();

            // Load the file using an XmlDocument
            XmlDocument xml = new XmlDocument();
            xml.Load(this.ItemsFileName);

            // Select the config nodes
            XmlNode configNode = xml.SelectSingleNode("/dock/config");

            if (configNode == null)
            {
                this.AlwaysOnTop = true;
                this.ShowPopupDescription = true;
            }
            else
            {
                // Get the configuration options
                bool alwaysOnTop = true;

                if (configNode["alwaysOnTop"] != null)
                {
                    Boolean.TryParse(configNode["alwaysOnTop"].InnerText, out alwaysOnTop);
                }

                bool showPopupDescription = true;

                if (configNode["showPopupDescription"] != null)
                {
                    Boolean.TryParse(configNode["showPopupDescription"].InnerText, out showPopupDescription);
                }

                // Update the properties
                this.AlwaysOnTop = alwaysOnTop;
                this.ShowPopupDescription = showPopupDescription;
            }
                        
            // Select the item nodes
            XmlNodeList itemNodes = xml.SelectNodes("/dock/items/item");

            foreach (XmlNode itemNode in itemNodes)
            {
                // Get the item number and fileName
                int itemNumber = Int32.Parse(itemNode.Attributes["number"].Value);
                int posX = 0;
                string fileName = itemNode["fileName"].InnerText;

                if (itemNode.Attributes["X"] != null)
                {
                    Int32.TryParse(itemNode.Attributes["X"].Value, out posX);
                }

                // Add the DockItem to this collection
                this.Add(DockItem.Load(itemNumber, fileName, posX));
            }
        }

        #endregion

        #region Private methods

        /// <summary>
        /// Fires the OnItemAdded event
        /// </summary>
        /// <param name="item"></param>
        private void ItemAdded(DockItem item)
        {
            if (this.OnItemAdded != null)
            {
                // Get the items index
                int itemIndex = this.innerDockItems.IndexOf(item);

                // Fire the OnItemAdded event
                this.OnItemAdded(this, new DockItemEventArgs(itemIndex, item));
            }
        }

        /// <summary>
        /// Fires the OnItemRemoved event
        /// </summary>
        /// <param name="item"></param>
        private void ItemRemoved(DockItem item)
        {
            if (this.OnItemRemoved != null)
            {
                // Get the items index
                int itemIndex = this.innerDockItems.IndexOf(item);

                // Fire the OnItemRemoved event
                this.OnItemRemoved(this, new DockItemEventArgs(itemIndex, item));
            }
        }
                
        #endregion
    }
}
