﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Text;
using System.Windows.Forms;

namespace Dock
{
    public partial class FormDock : Form
    {
        #region Fields

        /// <summary>
        /// Get the maximum number of visible DocItems
        /// </summary>
        private int maxVisibleDocItems;

        /// <summary>
        /// Collection of the items in the doc
        /// </summary>
        private Dock innerDock;

        /// <summary>
        /// Opacity of the form when its active
        /// </summary>
        private double opacityHover;

        /// <summary>
        /// Opacity of the form when its dim
        /// </summary>
        private double opacityDim;

        /// <summary>
        /// Is an object being dragged
        /// </summary>
        private bool isDragging;

        /// <summary>
        /// Current mouse coords
        /// </summary>
        private int currentX, currentY;

        #endregion

        #region Constructors

        public FormDock()
        {
            InitializeComponent();

            // Initialise the notification icon
            this.notifyIconDock.Text = Application.ProductName;

            // Calculate the maximum number of doc items which can be displayed
            this.maxVisibleDocItems = (Screen.PrimaryScreen.WorkingArea.Width / 64);

            // Initialise the inner dock items
            this.innerDock = new Dock(this.maxVisibleDocItems);
            this.innerDock.OnItemAdded += new EventHandler<DockItemEventArgs>(innerDockItems_OnItemAdded);
            this.innerDock.OnItemRemoved += new EventHandler<DockItemEventArgs>(innerDockItems_OnItemRemoved);
            this.innerDock.Load();

            // Update the menu items
            this.menuDockShowPopup.Checked = this.innerDock.ShowPopupDescription;
            this.menuDockAlwaysOnTop.Checked = this.innerDock.AlwaysOnTop;

            // Set the background color to black
            this.BackColor = Color.Black;

            // Set the form opactity values
            this.opacityHover = 0.7;     // 80%
            this.opacityDim = 0.7;      // 50%
            this.Opacity = this.opacityDim;

            // Dock the dock!
            this.Left = 0;
            this.Top = (Screen.PrimaryScreen.WorkingArea.Height - this.Height);
            this.Width = (Screen.PrimaryScreen.WorkingArea.Width);
        }

        #endregion

        #region DockItem handlers

        /// <summary>
        /// Fired when an item is removed from the Dock
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void innerDockItems_OnItemRemoved(object sender, DockItemEventArgs e)
        {
            // Calculate the name of the picture box to remove
            string picBoxName = "picture" + (e.Item.Index - 1);

            // Remove the picture box
            for (int i = 0; i < this.Controls.Count; i++)
            {
                if (this.Controls[i] is PictureBox)
                {
                    if (this.Controls[i].Name == picBoxName)
                    {
                        // Remove the picture box
                        this.Controls.RemoveAt(i);

                        // Delete the dock item file
                        e.Item.Delete();
                    }
                }
            }
        }

        /// <summary>
        /// Fired when an item is added to the Dock
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void innerDockItems_OnItemAdded(object sender, DockItemEventArgs e)
        {
            // Create a new picture
            PictureBox picBox = new PictureBox();
            picBox.Left = (e.Index * 64);            
            picBox.Top = 0;
            picBox.Width = 64;
            picBox.Height = 64;
            picBox.Name = "picture" + (e.Item.Index - 1);
            picBox.SizeMode = PictureBoxSizeMode.CenterImage;
            picBox.Tag = e.Item;
            picBox.Click += new EventHandler(picBox_Click);
            picBox.DoubleClick += new EventHandler(picBox_DoubleClick);
            picBox.MouseDown += new MouseEventHandler(picBox_MouseDown);
            picBox.MouseMove += new MouseEventHandler(picBox_MouseMove);
            picBox.MouseUp += new MouseEventHandler(picBox_MouseUp);
            picBox.MouseHover += new EventHandler(picBox_MouseHover);
            picBox.MouseLeave += new EventHandler(picBox_MouseLeave);

            if (e.Item.PositionX > 0)
            {
                // Set the Left position of the picture box
                picBox.Left = e.Item.PositionX;
            }

            // Get the image for the icon
            Icon ico = e.Item.GetLargeIcon();

            if (ico == null)
            {
                picBox.Image = Properties.Resources.image_web_layout_32;
            }
            else
            {
                picBox.Image = ico.ToBitmap();
            }

            // Update the DockItem
            e.Item.PositionX = picBox.Left;

            // Add the picBox to the form
            this.Controls.Add(picBox);

            // Set the tooltip
            this.toolTipImages.SetToolTip(picBox, e.Item.Tooltip);
        }

        void picBox_MouseUp(object sender, MouseEventArgs e)
        {
            if (sender is PictureBox)
            {
                // Get the PictureBox
                PictureBox picBox = (PictureBox)sender;

                // Set the isDragging flag to false
                this.isDragging = false;

                if (picBox.Top < -64)
                {
                    // Remove the item
                    this.innerDock.Remove((DockItem)picBox.Tag);
                }
                else
                {
                    // Set the top position to zero
                    picBox.Top = 0;

                    if (picBox.Left < 0)
                    {
                        picBox.Left = 0;
                    }

                    if (picBox.Left > (Screen.PrimaryScreen.WorkingArea.Width - picBox.Width))
                    {
                        picBox.Left = (Screen.PrimaryScreen.WorkingArea.Width - picBox.Width);
                    }

                    // Save the items file
                    this.innerDock.BackgroundSave();
                }
            }
        }

        void picBox_MouseMove(object sender, MouseEventArgs e)
        {
            if (this.isDragging)
            {
                if (sender is PictureBox)
                {
                    // Get the PictureBox
                    PictureBox picBox = (PictureBox)sender;

                    if (picBox.Tag is DockItem)
                    {
                        // Get the DockItem
                        DockItem dockItem = (DockItem)picBox.Tag;

                        // Move the picture box
                        picBox.Top = picBox.Top + (e.Y - this.currentY);
                        picBox.Left = picBox.Left + (e.X - this.currentX);

                        // Update the dockItem
                        dockItem.PositionX = picBox.Left;
                    }
                }
            }
        }

        void picBox_MouseDown(object sender, MouseEventArgs e)
        {
            this.isDragging = true;
            this.currentX = e.X;
            this.currentY = e.Y;
        }

        void picBox_DragDrop(object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent(DataFormats.FileDrop))
            {
                e.Effect = DragDropEffects.All;

                PictureBox picBox = (PictureBox)sender;
                picBox.DoDragDrop(picBox, DragDropEffects.All);
            }
        }

        void picBox_MouseLeave(object sender, EventArgs e)
        {
            if (sender is PictureBox)
            {
                // Set the forms opacity to the opacityDim value
                this.Opacity = this.opacityDim;

                // Get the DockItem
                PictureBox picBox = (PictureBox)sender;

                // Change the SizeMode of the picBoc to CenterImage
                picBox.SizeMode = PictureBoxSizeMode.CenterImage;

                if (picBox.Tag is DockItem)
                {
                    // Get the DockItem
                    DockItem dockItem = (DockItem)picBox.Tag;
                }
            }
        }

        void picBox_MouseHover(object sender, EventArgs e)
        {
            if (sender is PictureBox)
            {
                // Set the forms opacity to the opacityHover value
                this.Opacity = this.opacityHover;

                // Get the DockItem
                PictureBox picBox = (PictureBox)sender;

                // Change the SizeMode of the picBoc to StretchImage
                picBox.SizeMode = PictureBoxSizeMode.StretchImage;

                if (picBox.Tag is DockItem)
                {
                    // Get the DockItem
                    DockItem dockItem = (DockItem)picBox.Tag;
                }
            }
        }

        void picBox_Click(object sender, EventArgs e)
        {
            if (sender is PictureBox)
            {
                // Get the PictureBox
                PictureBox picBox = (PictureBox)sender;

                
            }
        }

        void picBox_DoubleClick(object sender, EventArgs e)
        {
            if (sender is PictureBox)
            {
                // Get the PictureBox
                PictureBox picBox = (PictureBox)sender;
                
                if (picBox.Tag is DockItem)
                {
                    // Get the DockItem
                    DockItem dockItem = (DockItem)picBox.Tag;

                    // Try and open the DockItem
                    try
                    {
                        dockItem.Open();
                    }
                    catch (FileNotFoundException fileEx)
                    {
                        // Show a message box
                        MessageBox.Show(fileEx.Message,
                            Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    }
                }
            }
        }

        #endregion

        private void FormDock_Load(object sender, EventArgs e)
        {
            
        }

        private void FormDock_MouseClick(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Right)
            {
                
            }
        }

        #region Form drag and drop events

        private void FormDock_DragDrop(object sender, DragEventArgs e)
        {
            // Get the array of files dropped
            string[] filesArray = (string[])e.Data.GetData(DataFormats.FileDrop, false);
            
            foreach (string fileName in filesArray)
            {
                if (!this.innerDock.IsFull)
                {
                    // Add the files to the innerDockItems collection
                    this.innerDock.Add(DockItem.Create(this.innerDock.Count, fileName, e.X));
                }
                else
                {
                    // Show a message box
                    MessageBox.Show("The dock is full, no further icons can be added",
                        Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                }
            }
        }

        private void FormDock_DragEnter(object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent(DataFormats.FileDrop))
            {
                e.Effect = DragDropEffects.Link;
            }
            else
            {
                e.Effect = DragDropEffects.None;
            }
        }

        #endregion

        private void menuDockExit_Click(object sender, EventArgs e)
        {
            // Close the form
            this.Close();
        }

        private void FormDock_FormClosing(object sender, FormClosingEventArgs e)
        {
            // Update the config file
            this.innerDock.AlwaysOnTop = this.menuDockAlwaysOnTop.Checked;
            this.innerDock.ShowPopupDescription = this.menuDockShowPopup.Checked;

            // Save the items file
            this.innerDock.Save();
        }

        private void alwaysOnTopToolStripMenuItem_Click(object sender, EventArgs e)
        {
            // Update the configuration file
            this.innerDock.AlwaysOnTop = menuDockAlwaysOnTop.Checked;

            if (menuDockAlwaysOnTop.Checked)
            {
                this.menuDockAlwaysOnTop.Checked = false;
                this.TopMost = false;
            }
            else
            {
                menuDockAlwaysOnTop.Checked = true;
                this.TopMost = true;
            }
        }

        private void FormDock_MouseMove(object sender, MouseEventArgs e)
        {
            // Set the current mouse position
            
        }

        private void menuDockShowPopup_Click(object sender, EventArgs e)
        {
            // Update the configuration file
            this.innerDock.ShowPopupDescription = menuDockShowPopup.Checked;

            if (this.menuDockShowPopup.Checked)
            {
                this.menuDockShowPopup.Checked = false;
                this.toolTipImages.Active = false;
            }
            else
            {
                this.menuDockShowPopup.Checked = true;
                this.toolTipImages.Active = true;
            }
        }


    }
}
