﻿using System;
using System.Diagnostics;
using System.Runtime.InteropServices;
using System.Windows.Forms;

namespace DockConsole
{
    public class Win32Keyboard
    {
        #region Events

        /// <summary>
        /// Fired when a key is pressed
        /// </summary>
        public event EventHandler<KeyEventArgs> OnKeyPress;
        
        #endregion

        #region Fields

        private const int WH_KEYBOARD_LL = 13;
        private const int WM_KEYDOWN = 0x0100;
        private const int WM_SYSKEYDOWN = 0x0104;
        private LowLevelKeyboardProc _proc;
        private IntPtr _hookID;

        #endregion

        #region Constructors

        /// <summary>
        /// Windows keyboard listener
        /// </summary>
        public Win32Keyboard()
        {
            this._hookID = IntPtr.Zero;
        }

        #endregion

        #region Public methods

        /// <summary>
        /// Start grabbing keypresses
        /// </summary>
        public void Start()
        {
            this._proc = this.HookCallback;
            this._hookID = this.SetHook(this._proc);
        }

        /// <summary>
        /// Stop grabbing keypresses
        /// </summary>
        public void Stop()
        {
            this.UnsetHook(this._hookID);
        }

        #endregion

        #region Private methods

        /// <summary>
        /// Set a keyboard hook
        /// </summary>
        /// <param name="proc"></param>
        /// <returns></returns>
        private IntPtr SetHook(LowLevelKeyboardProc proc)
        {
            using (Process curProcess = Process.GetCurrentProcess())
            using (ProcessModule curModule = curProcess.MainModule)
            {
                return SetWindowsHookEx(WM_SYSKEYDOWN, proc,
                    GetModuleHandle(curModule.ModuleName), 0);
            }
        }

        /// <summary>
        /// Unset a keyboard hook
        /// </summary>
        /// <param name="ptr"></param>
        private bool UnsetHook(IntPtr ptr)
        {
            return UnhookWindowsHookEx(ptr);
        }

        private delegate IntPtr LowLevelKeyboardProc(int nCode, IntPtr wParam, IntPtr lParam);

        /// <summary>
        /// Fired when a keypress is received from Windows
        /// </summary>
        /// <param name="nCode"></param>
        /// <param name="wParam"></param>
        /// <param name="lParam"></param>
        /// <returns></returns>
        private IntPtr HookCallback(int nCode, IntPtr wParam, IntPtr lParam)
        {
            if (nCode >= 0 && wParam == (IntPtr)WM_KEYDOWN)
            {
                int vkCode = Marshal.ReadInt32(lParam);

                // Fire the OnKeyPress event
                if (this.OnKeyPress != null)
                {
                    this.OnKeyPress(null, new KeyEventArgs((Keys)vkCode));
                }
            }
            return CallNextHookEx(_hookID, nCode, wParam, lParam);
        }

        #endregion

        #region Dll imports

        [DllImport("user32.dll", CharSet = CharSet.Auto, SetLastError = true)]
        private static extern IntPtr SetWindowsHookEx(int idHook,
            LowLevelKeyboardProc lpfn, IntPtr hMod, uint dwThreadId);

        [DllImport("user32.dll", CharSet = CharSet.Auto, SetLastError = true)]
        [return: MarshalAs(UnmanagedType.Bool)]
        private static extern bool UnhookWindowsHookEx(IntPtr hhk);

        [DllImport("user32.dll", CharSet = CharSet.Auto, SetLastError = true)]
        private static extern IntPtr CallNextHookEx(IntPtr hhk, int nCode,
            IntPtr wParam, IntPtr lParam);

        [DllImport("kernel32.dll", CharSet = CharSet.Auto, SetLastError = true)]
        private static extern IntPtr GetModuleHandle(string lpModuleName);

        #endregion
    }
}
